﻿using MapInfo.RasterEngine.IO;
using MapInfo.RasterEngine.Common;
using System;
using System.Collections.Generic;
using System.Drawing;

namespace GetRasterCellValue
{
	class Program
	{/// <summary>
    /// It will allow user to get cell value of each band under specified field and cell coordinates.
    /// </summary>
    /// <param name="args"></param>
		static void Main(string[] args)
		{
		    long cellX = 4;
		    long cellY = 4;
		    uint nField = 0;
		    string inputFilePath = System.IO.Path.GetFullPath(@"..\..\..\Input\12_ClassTable_AllDataType.mrr");
		    IRasterDataset dataset = null;
            try
			{
				//Open Raster file
				dataset = RasterDatasetFactory.Open(inputFilePath);

			    for (uint bandIndex = 0; bandIndex < dataset.Info.FieldInfo[(int)nField].BandInfo.Count; bandIndex++)
			    {
			        IRasterBand datasetBand = dataset.GetBand(nField, bandIndex);

			        Console.Write("Band#{0}[{1}]:",bandIndex, datasetBand.Info.Name);

                    #region Random Iterator implementation
                    RasterRandomIterator randomIteratorRead = dataset.GetRandomIterator(nField, null, null, 0, IteratorMode.Read);
                    randomIteratorRead.Begin();
                    IRandomBand iRandomBand = randomIteratorRead[bandIndex];

                    bool bRet = false;
                    bool bValid = false;
                    switch (dataset.Info.FieldInfo[(int)nField].BandInfo[(int)bandIndex].DataType)
                    {
                        case RasterBandDataType.Red:
                            {
                                byte outCellValue = byte.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    Color clrArgb = Color.FromArgb(outCellValue, 0, 0);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = Red[{2}]", cellX, cellY, clrArgb.R);
                                }
                                break;
                            }
                        case RasterBandDataType.Green:
                            {
                                byte outCellValue = byte.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    Color clrArgb = Color.FromArgb(0, outCellValue, 0);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = Green[{2}]", cellX, cellY, clrArgb.G);
                                }
                                break;
                            }
                        case RasterBandDataType.Blue:
                            {
                                byte outCellValue = byte.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    Color clrArgb = Color.FromArgb(0, 0, outCellValue);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = Blue[{2}]", cellX, cellY, clrArgb.B);
                                }
                                break;
                            }
                        case RasterBandDataType.Alpha:
                            {
                                byte outCellValue = byte.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    Color clrArgb = Color.FromArgb(outCellValue, 0, 0, 0);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = Alpha[{2}]", cellX, cellY, clrArgb.A);
                                }
                                break;
                            }
                        case RasterBandDataType.BlackGrey8:
                        case RasterBandDataType.WhiteGrey8:
                            {
                                byte outCellValue = byte.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    Color clrArgb = Color.FromArgb(outCellValue, outCellValue, outCellValue);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = RGB[{2},{3},{4}]", cellX, cellY, clrArgb.B, clrArgb.G, clrArgb.R);
                                }
                                break;
                            }
                        case RasterBandDataType.RedAlpha:
                            {
                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[1], byteArray[0], 0, 0);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = RA[{2},{3}]", cellX, cellY, clrArgb.R, clrArgb.A);
                                }
                                break;
                            }
                        case RasterBandDataType.GreenAlpha:
                            {
                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[1], 0, byteArray[0], 0);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = GA[{2},{3}]", cellX, cellY, clrArgb.G, clrArgb.A);
                                }
                                break;
                            }
                        case RasterBandDataType.BlueAlpha:
                            {
                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[1], 0, 0, byteArray[0]);
                                    Console.WriteLine(
                                        "Cell[X={0}, Y={1}] = BA[{2},{3}]", cellX, cellY, clrArgb.B, clrArgb.A);
                                }
                                break;
                            }
                        case RasterBandDataType.GreyAlpha:
                            {
                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[1], byteArray[0], byteArray[0], byteArray[0]);
                                    Console.WriteLine(
                                        "Cell[X={0}, Y={1}] = RGBA[{2},{3},{4},{5}]", cellX, cellY, clrArgb.R, clrArgb.G, clrArgb.B, clrArgb.A);
                                }
                                break;
                            }
                        case RasterBandDataType.SignedInt8:
                            {
                                sbyte outCellValue = sbyte.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] = {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.SignedInt16:
                            {
                                Int16 outCellValue = Int16.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.SignedInt32:
                            {
                                Int32 outCellValue = Int32.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.SignedInt64:
                            {
                                Int64 outCellValue = Int64.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.Grey:
                        case RasterBandDataType.UnsignedInt8:
                            {
                                byte outCellValue = byte.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.UnsignedInt16:
                            {
                                UInt16 outCellValue = UInt16.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.UnsignedInt32:
                            {
                                UInt32 outCellValue = UInt32.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.UnsignedInt64:
                            {
                                UInt64 outCellValue = UInt64.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.RealSingle:
                            {
                                float outCellValue = float.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.RealDouble:
                            {
                                double outCellValue = double.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid) Console.WriteLine("Cell[X={0}, Y={1}] =  {2}", cellX, cellY, outCellValue);
                                break;
                            }
                        case RasterBandDataType.BlueGreenRed:
                            {

                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[2], byteArray[1], byteArray[0]);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = BGR[{2},{3},{4}]", cellX, cellY, clrArgb.B, clrArgb.G, clrArgb.R);
                                }
                                break;
                            }
                        case RasterBandDataType.BlueGreenRedAlpha:
                            {
                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[3], byteArray[2], byteArray[1], byteArray[0]);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = BGRA[{2},{3},{4},{5}]", cellX, cellY, clrArgb.B, clrArgb.G, clrArgb.R, clrArgb.A);
                                }
                                break;
                            }
                        case RasterBandDataType.RedGreenBlue:
                            {
                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[0], byteArray[1], byteArray[2]);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = RGB[{2},{3},{4}]", cellX, cellY, clrArgb.R, clrArgb.G, clrArgb.B);
                                }
                                break;
                            }
                        case RasterBandDataType.RedGreenBlueAlpha:
                            {
                                int outCellValue = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outCellValue, out bValid);
                                if (bRet && bValid)
                                {
                                    byte[] byteArray = BitConverter.GetBytes(outCellValue);
                                    Color clrArgb = Color.FromArgb(byteArray[3], byteArray[2], byteArray[1], byteArray[0]);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = RGBA[{2},{3},{4},{5}]", cellX, cellY, clrArgb.R, clrArgb.G, clrArgb.B, clrArgb.A);
                                }
                                break;
                            }
                        case RasterBandDataType.UTF32String:
                        case RasterBandDataType.UTF16String:
                        case RasterBandDataType.AsciiString:
                        case RasterBandDataType.UTF8String:
                            {
                                // Set first band of classified raster
                                iRandomBand = randomIteratorRead[0];
                                int outClassIndex = int.MaxValue;
                                bRet = iRandomBand.GetCellValue(cellX, cellY, out outClassIndex, out bValid);
                                if (bRet && bValid)
                                {
                                    // Get classification Table records
                                    IList<ClassTableRecord> records = dataset.ClassTables[(int)nField].TableRecords;
                                    string outCellValue = Convert.ToString(records[outClassIndex].RecordFields[(int)bandIndex - 1]);
                                    Console.WriteLine("Cell[X={0}, Y={1}] = '{2}'", cellX, cellY, outCellValue);
                                }
                                break;
                            }
                        default:
                                Console.WriteLine("Band Data type '{0}' is not defined.",
                                    dataset.Info.FieldInfo[(int)nField].BandInfo[(int)bandIndex].DataType);
                                break;
                    }
                    if (!bRet)
                        Console.WriteLine("** Unable to get CellValue of raster.");
                    #endregion
                }
			}
			catch(Exception ex)
			{
			    if (ex.Message == "Error: Iterator failed.")
			    {
			        Console.WriteLine("Cell[X={0}, Y={1}] = No Cell", cellX, cellY);
                }
			    else
			    {
			        Console.WriteLine("Exception encountered: {0}", ex.Message);
                }
			}
			finally
			{
                //Close the raster
				if (dataset != null) dataset.Close();
				
			}
			Console.ReadLine();
		}
    }

}
